@@ -21,8 +21,6 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 
-from __future__ import print_function
-
 import gettext
 
 import locale
@@ -62,8 +60,13 @@ try:
 except AttributeError:
     pass
 
-gettext.bindtextdomain(DOMAINNAME, localedir)
-gettext.bind_textdomain_codeset(DOMAINNAME, 'UTF-8')
+# Python's gettext module doesn't provide all methods in
+# new Python version
+try:
+    gettext.bindtextdomain(DOMAINNAME, localedir)
+    gettext.bind_textdomain_codeset(DOMAINNAME, 'UTF-8')
+except AttributeError:
+    pass
 
 gettext.install(DOMAINNAME, localedir)
 
@@ -75,7 +78,7 @@ class PreferencesDialog:
         self.__builder.add_from_file("ibus-libpinyin-preferences.ui")
         self.__dialog = self.__builder.get_object("dialog")
         self.__init_pages()
-        
+
         if engine == "libpinyin":
             self.__config_namespace = "com.github.libpinyin.ibus-libpinyin.libpinyin"
             self.__config = Gio.Settings.new(self.__config_namespace)
@@ -85,6 +88,7 @@ class PreferencesDialog:
             self.__init_dictionary()
             self.__init_user_data()
             self.__init_shortcut()
+            self.__init_cloud_input()
             self.__init_about()
         elif engine == "libbopomofo":
             self.__config_namespace = "com.github.libpinyin.ibus-libpinyin.libbopomofo"
@@ -95,8 +99,10 @@ class PreferencesDialog:
             self.__init_dictionary()
             #self.__init_user_data()
             self.__init_shortcut()
+            self.__init_bopomofo_cloud_input()
             self.__init_about()
             self.__convert_fuzzy_pinyin_to_bopomofo()
+            self.__display_style_box.hide()
 
         else:
             print("Error: Unknown Engine")
@@ -138,12 +144,13 @@ class PreferencesDialog:
         self.__init_trad = self.__builder.get_object("InitTraditionalChinese")
 
         # UI
+        self.__display_style = self.__builder.get_object("DisplayStyle")
+        self.__display_style_box = self.__builder.get_object("boxDisplayStyle")
         self.__lookup_table_orientation = self.__builder.get_object("LookupTableOrientation")
         self.__lookup_table_page_size = self.__builder.get_object("LookupTablePageSize")
 
         self.__dynamic_adjust = self.__builder.get_object("DynamicAdjust")
         self.__remember_every_input = self.__builder.get_object("RememberEveryInput")
-        self.__show_suggestion = self.__builder.get_object("ShowSuggestion")
         self.__sort_candidate_option = self.__builder.get_object("SortCandidateOption")
 
         # read values
@@ -154,10 +161,10 @@ class PreferencesDialog:
 
         self.__lookup_table_orientation.set_active(self.__get_value("lookup-table-orientation"))
         self.__lookup_table_page_size.set_value(self.__get_value("lookup-table-page-size"))
+        self.__display_style.set_active(self.__get_value("display-style"))
 
         self.__dynamic_adjust.set_active(self.__get_value("dynamic-adjust"))
         self.__remember_every_input.set_active(self.__get_value("remember-every-input"))
-        self.__show_suggestion.set_active(self.__get_value("show-suggestion"))
         self.__sort_candidate_option.set_active(self.__get_value("sort-candidate-option"))
         # connect signals
         self.__init_chinese.connect("toggled", self.__toggled_cb, "init-chinese")
@@ -166,17 +173,20 @@ class PreferencesDialog:
         self.__init_simp.connect("toggled", self.__toggled_cb, "init-simplified-chinese")
         self.__dynamic_adjust.connect("toggled", self.__toggled_cb, "dynamic-adjust")
         self.__remember_every_input.connect("toggled", self.__toggled_cb, "remember-every-input")
-        self.__show_suggestion.connect("toggled", self.__toggled_cb, "show-suggestion")
 
-        def __lookup_table_page_size_changed_cb(adjustment):
-            self.__set_value("lookup-table-page-size", int(adjustment.get_value()))
+        def __display_size_changed_cb(widget):
+            self.__set_value("display-style", widget.get_active())
 
         def __lookup_table_orientation_changed_cb(widget):
             self.__set_value("lookup-table-orientation", widget.get_active())
 
+        def __lookup_table_page_size_changed_cb(adjustment):
+            self.__set_value("lookup-table-page-size", int(adjustment.get_value()))
+
         def __sort_candidate_option_changed_cb(widget):
             self.__set_value("sort-candidate-option", widget.get_active())
 
+        self.__display_style.connect("changed", __display_size_changed_cb)
         self.__lookup_table_orientation.connect("changed", __lookup_table_orientation_changed_cb)
         self.__lookup_table_page_size.connect("value-changed", __lookup_table_page_size_changed_cb)
         self.__sort_candidate_option.connect("changed", __sort_candidate_option_changed_cb)
@@ -184,7 +194,7 @@ class PreferencesDialog:
     def __init_pinyin(self):
         # page
         self.__page_pinyin_mode.show()
-        
+
         # pinyin
         self.__full_pinyin = self.__builder.get_object("FullPinyin")
         self.__incomplete_pinyin = self.__builder.get_object("IncompletePinyin")
@@ -192,12 +202,12 @@ class PreferencesDialog:
         self.__double_pinyin_schema = self.__builder.get_object("DoublePinyinSchema")
         # self.__double_pinyin_schema_label = self.__builder.get_object("labelDoublePinyinSchema")
         self.__double_pinyin_show_raw = self.__builder.get_object("DoublePinyinShowRaw")
-        self.__double_pinyin_show_raw.hide ()
 
         # read value
         self.__incomplete_pinyin.set_active(self.__get_value("incomplete-pinyin"))
         self.__full_pinyin.set_active(not self.__get_value("double-pinyin"))
         self.__double_pinyin_schema.set_active(self.__get_value("double-pinyin-schema"))
+        self.__double_pinyin_show_raw.set_active(self.__get_value("double-pinyin-show-raw"))
         if self.__full_pinyin.get_active():
             # self.__incomplete_pinyin.set_sensitive(True)
             self.__double_pinyin_schema.set_sensitive(False)
@@ -235,7 +245,7 @@ class PreferencesDialog:
         # bopomofo mode
         self.__incomplete_bopomofo = self.__builder.get_object("IncompleteBopomofo")
         self.__bopomofo_keyboard_mapping = self.__builder.get_object("BopomofoKeyboardMapping")
-        
+
         # selection mode
         self.__select_keys = self.__builder.get_object("SelectKeys")
         self.__guide_key = self.__builder.get_object("GuideKey")
@@ -343,12 +353,13 @@ class PreferencesDialog:
                 self.__builder.get_object(w[0]).set_sensitive(val)
         self.__fuzzy_pinyin.connect("toggled", __fuzzy_pinyin_toggled_cb)
 
-        # init value
+        # read values
         self.__fuzzy_pinyin.set_active(self.__get_value("fuzzy-pinyin"))
         for name, keyname in self.__fuzzy_pinyin_widgets:
             widget = self.__builder.get_object(name)
             widget.set_active(self.__get_value(keyname))
 
+        # connect signals
         self.__fuzzy_pinyin.connect("toggled", self.__toggled_cb, "fuzzy-pinyin")
         for name, keyname in self.__fuzzy_pinyin_widgets:
             widget = self.__builder.get_object(name)
@@ -395,20 +406,58 @@ class PreferencesDialog:
         path = os.path.join(pkgdatadir, 'user.lua')
         if not os.access(path, os.R_OK):
             self.__frame_lua_script.hide()
-
+        self.__frame_user_table = self.__builder.get_object("frameUserTable")
+        self.__lua_extension = self.__builder.get_object("LuaExtension")
+        self.__table_mode = self.__builder.get_object("TableMode")
+        self.__english_mode = self.__builder.get_object("EnglishMode")
+        self.__emoji_candidate = self.__builder.get_object("EmojiCandidate")
+        self.__english_candidate = self.__builder.get_object("EnglishCandidate")
+        self.__suggestion_candidate = self.__builder.get_object("SuggestionCandidate")
+        self.__import_table = self.__builder.get_object("ImportTable")
+        self.__export_table = self.__builder.get_object("ExportTable")
+        self.__clear_user_table = self.__builder.get_object("ClearUserTable")
         self.__edit_lua = self.__builder.get_object("EditLua")
-        self.__edit_lua.connect("clicked", self.__edit_lua_cb)
-
         self.__import_dictionary = self.__builder.get_object("ImportDictionary")
-        self.__import_dictionary.connect("clicked", self.__import_dictionary_cb)
-
         self.__export_dictionary = self.__builder.get_object("ExportDictionary")
-        self.__export_dictionary.connect("clicked", self.__export_dictionary_cb)
+        self.__clear_user_data = self.__builder.get_object("ClearUserDictionary")
+        self.__clear_all_data = self.__builder.get_object("ClearAllDictionary")
 
-        self.__clear_user_data = self.__builder.get_object("ClearUserData")
+        # read values
+        self.__frame_lua_script.set_sensitive(self.__get_value("lua-extension"))
+        self.__frame_user_table.set_sensitive(self.__get_value("table-input-mode"))
+        self.__lua_extension.set_active(self.__get_value("lua-extension"))
+        self.__table_mode.set_active(self.__get_value("table-input-mode"))
+        self.__english_mode.set_active(self.__get_value("english-input-mode"))
+        self.__emoji_candidate.set_active(self.__get_value("emoji-candidate"))
+        self.__english_candidate.set_active(self.__get_value("english-candidate"))
+        self.__suggestion_candidate.set_active(self.__get_value("suggestion-candidate"))
+
+        # connect signals
+        self.__lua_extension.connect("toggled", self.__lua_extension_cb)
+        self.__table_mode.connect("toggled", self.__table_mode_cb)
+        self.__english_mode.connect("toggled", self.__english_mode_cb)
+        self.__emoji_candidate.connect("toggled", self.__toggled_cb, "emoji-candidate")
+        self.__english_candidate.connect("toggled", self.__toggled_cb, "english-candidate")
+        self.__suggestion_candidate.connect("toggled", self.__toggled_cb, "suggestion-candidate")
+        self.__edit_lua.connect("clicked", self.__edit_lua_cb)
+        self.__import_dictionary.connect("clicked", self.__import_dictionary_cb, "import-dictionary")
+        self.__export_dictionary.connect("clicked", self.__export_dictionary_cb, "export-dictionary")
         self.__clear_user_data.connect("clicked", self.__clear_user_data_cb, "user")
-        self.__clear_all_data = self.__builder.get_object("ClearAllData")
         self.__clear_all_data.connect("clicked", self.__clear_user_data_cb, "all")
+        self.__import_table.connect("clicked", self.__import_table_cb, "import-custom-table")
+        self.__export_table.connect("clicked", self.__export_table_cb, "export-custom-table")
+        self.__clear_user_table.connect("clicked", self.__clear_user_table_cb, "clear-custom-table", "user")
+
+    def __lua_extension_cb(self, widget):
+        self.__set_value("lua-extension", widget.get_active())
+        self.__frame_lua_script.set_sensitive(widget.get_active())
+
+    def __table_mode_cb(self, widget):
+        self.__set_value("table-input-mode", widget.get_active())
+        self.__frame_user_table.set_sensitive(widget.get_active())
+
+    def __english_mode_cb(self, widget):
+        self.__set_value("english-input-mode", widget.get_active())
 
     def __edit_lua_cb(self, widget):
         import shutil
@@ -420,30 +469,35 @@ class PreferencesDialog:
             shutil.copyfile(src, path)
         os.system("xdg-open %s" % path)
 
-    def __import_dictionary_cb(self, widget):
+    def __get_import_filename(self):
         dialog = Gtk.FileChooserDialog \
-            (_("Please choose a file"), self.__dialog,
-             Gtk.FileChooserAction.OPEN,
-             (Gtk.STOCK_CANCEL, Gtk.ResponseType.CANCEL,
-              Gtk.STOCK_OPEN, Gtk.ResponseType.OK))
+            (title = _("Please choose a file"), parent = self.__dialog,
+             action = Gtk.FileChooserAction.OPEN)
+
+        dialog.add_buttons(Gtk.STOCK_CANCEL, Gtk.ResponseType.CANCEL,
+                           Gtk.STOCK_OPEN, Gtk.ResponseType.OK)
 
         filter_text = Gtk.FileFilter()
         filter_text.set_name("Text files")
         filter_text.add_mime_type("text/plain")
         dialog.add_filter(filter_text)
 
+        filename = None
         response = dialog.run()
         if response == Gtk.ResponseType.OK:
-            self.__set_value("import-dictionary", dialog.get_filename())
-
+            filename = dialog.get_filename()
         dialog.destroy()
 
-    def __export_dictionary_cb(self, widget):
+        return filename
+
+    def __get_export_filename(self):
         dialog = Gtk.FileChooserDialog \
-                 (_("Please save a file"), self.__dialog,
-                  Gtk.FileChooserAction.SAVE,
-                  (Gtk.STOCK_CANCEL, Gtk.ResponseType.CANCEL,
-                   Gtk.STOCK_SAVE, Gtk.ResponseType.OK))
+                 (title = _("Please save a file"), parent = self.__dialog,
+                  action = Gtk.FileChooserAction.SAVE)
+
+        dialog.add_buttons(Gtk.STOCK_CANCEL, Gtk.ResponseType.CANCEL,
+                           Gtk.STOCK_SAVE, Gtk.ResponseType.OK)
+
         dialog.set_do_overwrite_confirmation(True)
 
         filter_text = Gtk.FileFilter()
@@ -451,15 +505,46 @@ class PreferencesDialog:
         filter_text.add_mime_type("text/plain")
         dialog.add_filter(filter_text)
 
+        filename = None
         response = dialog.run()
         if response == Gtk.ResponseType.OK:
-            self.__set_value("export-dictionary", dialog.get_filename())
-
+            filename = dialog.get_filename()
         dialog.destroy()
 
+        return filename
+
+    def __import_dictionary_cb(self, widget, name):
+        filename = self.__get_import_filename()
+        if filename:
+            self.__set_value(name, "")
+            self.__set_value(name, filename)
+
+    def __export_dictionary_cb(self, widget, name):
+        filename = self.__get_export_filename()
+        if filename:
+            self.__set_value(name, "")
+            self.__set_value(name, filename)
+
     def __clear_user_data_cb(self, widget, name):
         self.__set_value("clear-user-data", name)
 
+    def __import_table_cb(self, widget, name):
+        filename = self.__get_import_filename()
+        if filename:
+            self.__set_value(name, "")
+            self.__set_value(name, filename)
+            self.__set_value("use-custom-table", True)
+
+    def __export_table_cb(self, widget, name):
+        filename = self.__get_export_filename()
+        if filename:
+            self.__set_value(name, "")
+            self.__set_value(name, filename)
+
+    def __clear_user_table_cb(self, widget, name, value):
+        self.__set_value(name, value)
+        self.__set_value("use-custom-table", False)
+
     def __init_shortcut(self):
         # page Shortcut
         self.__page_shortcut.show()
@@ -479,6 +564,73 @@ class PreferencesDialog:
     def __shortcut_changed_cb(self, editor, key, value):
         self.__set_value(key, value)
 
+    def __init_bopomofo_cloud_input(self):
+        # init state
+        self.__frame_cloud_input = self.__builder.get_object("frameBopomofoCloudInput")
+        self.__init_enable_cloud_input = self.__builder.get_object("InitEnableBopomofoCloudInput")
+
+        # cloud input option
+        self.__cloud_input_source = self.__builder.get_object("BopomofoCloudInputSource")
+
+        if not config.support_cloud_input():
+            self.__frame_cloud_input.hide()
+
+        # read values
+        self.__init_enable_cloud_input.set_active(self.__get_value("enable-cloud-input"))
+
+        self.__cloud_input_source.set_active(self.__get_value("cloud-input-source"))
+
+        if self.__init_enable_cloud_input.get_active():
+            self.__cloud_input_source.set_sensitive(True)
+        else:
+            self.__cloud_input_source.set_sensitive(False)
+
+        # connect signals
+        def __enable_cloud_input_cb(widget):
+            val = widget.get_active()
+            self.__set_value("enable-cloud-input", val)
+            self.__cloud_input_source.set_sensitive(val)
+
+        def __cloud_input_source_changed_cb(widget):
+            self.__set_value("cloud-input-source", widget.get_active())
+
+        self.__init_enable_cloud_input.connect("toggled", __enable_cloud_input_cb)
+        self.__cloud_input_source.connect("changed", __cloud_input_source_changed_cb)
+
+
+    def __init_cloud_input(self):
+        # init state
+        self.__frame_cloud_input = self.__builder.get_object("frameCloudInput")
+        self.__init_enable_cloud_input = self.__builder.get_object("InitEnableCloudInput")
+
+        # cloud input option
+        self.__cloud_input_source = self.__builder.get_object("CloudInputSource")
+
+        if not config.support_cloud_input():
+            self.__frame_cloud_input.hide()
+
+        # read values
+        self.__init_enable_cloud_input.set_active(self.__get_value("enable-cloud-input"))
+
+        self.__cloud_input_source.set_active(self.__get_value("cloud-input-source"))
+
+        if self.__init_enable_cloud_input.get_active():
+            self.__cloud_input_source.set_sensitive(True)
+        else:
+            self.__cloud_input_source.set_sensitive(False)
+
+        # connect signals
+        def __enable_cloud_input_cb(widget):
+            val = widget.get_active()
+            self.__set_value("enable-cloud-input", val)
+            self.__cloud_input_source.set_sensitive(val)
+
+        def __cloud_input_source_changed_cb(widget):
+            self.__set_value("cloud-input-source", widget.get_active())
+
+        self.__init_enable_cloud_input.connect("toggled", __enable_cloud_input_cb)
+        self.__cloud_input_source.connect("changed", __cloud_input_source_changed_cb)
+
     def __init_about(self):
         # page About
         self.__page_about.show()
